/*
 * pda related functions for hfwload
 * (firmware loader for hermes wavelan cards)
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */
 
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include "pdar.h"

int verify_pda(unsigned char *data, int size) {
	unsigned char *pd=data;
	u16 len, code;
	int res=-1;
	
	while((pd-data)<size) {
		memcpy(&len,pd,sizeof(u16));
		len=((le16_to_host(len))<<1)-2;
		if(len>(size-4-(pd-data)))
			return -1;
		memcpy(&code,pd+sizeof(u16),sizeof(u16));
		code=le16_to_host(code);
		if(!code) {
			if(len==2)
				return 0;
			else
				return -1;
		}
		pd+=len+4;
	}
	return res;
}

int read_pda_from(int fd, off_t offset, unsigned char *data) { 
	int res;
	
	if(lseek(fd,offset,SEEK_SET)!=offset)
		return -1;
	else if((res=read(fd,data,PDA_SIZE_MAX))<6)
		return 1;
	else
		return verify_pda(data,res);
}

void dump_pda(unsigned char *data, int size) {
	unsigned char *pd=data;
	u16 len, code;
	int count;
	
	while((pd-data)<size) {
		memcpy(&len,pd,sizeof(u16));
		len=((le16_to_host(len))<<1)-2;
		if(len>(size-4-(pd-data)))
			return;
		memcpy(&code,pd+sizeof(u16),sizeof(u16));
		code=le16_to_host(code);
		if(!code) 
			return;
		printf("%04X: ",code);
		pd+=4;
		for(count=0;count<len;count+=2)
			printf("%02X%02X ",pd[count],pd[count+1]);
		printf("\n");
		pd+=len;
	}
}

unsigned char * lookup_pda (unsigned short code, unsigned short len, unsigned char *pd) {
	u16 len2, code2;
	
	if((!pd) || (!code) || (!len) || (len&1))
		return NULL;
	
	while(1) {
		memcpy(&len2,pd,sizeof(u16));
		len2=((le16_to_host(len2))<<1)-2;
		memcpy(&code2,pd+sizeof(u16),sizeof(u16));
		code2=le16_to_host(code2);
		if(!code2) 
			return NULL;
		pd+=4;
		//printf("%04X %04X %04X %04X\n",code,code2,len,len2);
		if((code2==code) && (len2==len))
			return pd;
		pd+=len2;
	}
	
	return NULL;
}